package dicewars.file;

import dicewars.model.Hex;
import javafx.scene.paint.Color;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/*
 * File IO, read/write hex grids from a text file
 *
 * The file format is:
 *
 * A single row with a unique Color String like 0xff9980ff,
 * then 1-n rows with hex coordinates separated by a "," like 0,3.
 * Then another unique color with coordinates etc. Like this:
 *
 * color1
 * hex coords
 * hex coords
 * ...
 * color2
 * hex coords
 * hex coords
 * ...
 */
public class FileService {

    public static final String SEPARATOR = ",";

    public static void writeHexGrid(Path path, Map<Color, List<Hex>> regions) throws IOException {
        try (BufferedWriter writer = Files.newBufferedWriter(path)) {
            for (Map.Entry<Color, List<Hex>> e : regions.entrySet()) {
                if (e.getValue().size() > 0) {
                    writer.write(e.getKey().toString());  // The color
                    writer.newLine();
                    for (Hex h : e.getValue()) {
                        writer.write(h.getXh() + SEPARATOR + h.getYh());  // The hexes
                        writer.newLine();
                    }
                }
            }
        }
    }

    // See exercises, see grid1.txt
    public static Map<Color, List<Hex>> readHexGrid(Path path) throws IOException {

        // TODO
        return null;
    }
}
