package samples;

import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.event.EventType;

import java.util.*;
import java.util.function.Consumer;

import static java.lang.System.out;

/**
 *  The use of method references
 *
 *  1. For event handling, similar to JavaFX. Just an illustration here.
 *  2. For internal iteration (forEach).
 *
 *
 *
    This is how it looks in JavaFX

    ParallelTransition dicesAnimation = ...
    dicesAnimation.setOnFinished(this::animationFinished);  // Method reference as argument

    private void animationFinished(Event e) {  // The method referenced
        from.setSelected(false);
        country.setSelected(false);
        renderCountry(from);
        renderCountry(country);
        renderPlayers();
        from = null;
    }


 */

public class MethodReferences {

    public static void main(String[] args) {
        new MethodReferences().program();
    }


    void program() {
        // This is similar to JavaFX use of method references
        A a = new A(this::handle);  // Method reference as argument
        a.accept(78);            // Call the stored method


        // Using method references to process collections

        List<A> as = Arrays.asList(new A(), new A(), new A(), new A(), new A());
        as.forEach(this::doIt);

        Map<String, A> sA = new HashMap<>();
        sA.values().forEach(this::doIt);

    }


    //  -------------- Method reference JavaFX style -----------------------------

    void handle(int i) {
        out.println(i);
    }  // The Consumer

    class A {
        Consumer<Integer> consumer;  // Variable for method reference
        String s = "aaa";

        A() {
        }

        A(Consumer<Integer> consumer) {
            this.consumer = consumer;
        }

        void accept(int i) {
            consumer.accept(i);
        }

    }

    // ---  Method reference for iteration ---------------------

    void doIt(A a) {
        out.println(a.s);
    }


}
