package samples.filehandling;

import jdk.nashorn.internal.runtime.options.Options;

import java.io.*;
import java.nio.channels.Channels;
import java.nio.channels.FileChannel;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import static java.lang.System.*;

/*
   Using binary files to store objects to disk and read them back into memory
 */
public class BinaryFiles {

    public static void main(String[] args) {
        new BinaryFiles().program();
    }

    void program() {
        // Object to store
        MyClass o = new MyClass("pelle");
        o.setName("pelle");
        try {
            Path path = Paths.get("src/samples/filehandling/pelle.ser" );
            writeObject(path, o);  // Write full object to file
            MyClass o1 = readObject(path);   // Read object back
            out.println(o.getName().equals(o1.getName())); // Same content
            out.println(o != o1);         // Other object!
        } catch (IOException | ClassNotFoundException  e) {
            e.printStackTrace();
        }
    }

    void writeObject(Path path, MyClass object) throws IOException {
        // Use try with resources to ensure stream is closed
        try (ObjectOutputStream os = new ObjectOutputStream(Files.newOutputStream(path))) {
            os.writeObject(object);
            os.close();
        }
    }

    MyClass readObject(Path path) throws IOException, ClassNotFoundException {
        try (ObjectInputStream is = new ObjectInputStream(Files.newInputStream(path));) {
            MyClass o = (MyClass) is.readObject();
            is.close();
            return o;
        }
    }

    // Must have static here for technical reasons (MyClass is an inner class)
    // More in later courses
    static class MyClass implements Serializable {
        private String name;

        public MyClass(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }
    }

}
